package HeroList;

import java.io.OutputStream;
import java.io.PrintStream;
import fi.jyu.mit.ohj2.Mjonot;

/**
 * A Hero who can, among other things, manage their own hId.
 * @author jessehonkalinna
 * @version Mar 11, 2025
 *
 */
public class Hero {
    /**
     * 
     */
    public int     hId; 
    /**
     * @return heroID
     */
    public int gethId() {
        return hId;
    }

    /**
     * @param hId asettaa HeroID
     */
    public void sethId(int hId) {
        this.hId = hId;
    }

    /**
     * @return hakee rodun
     */
    public String getRace() {
        return race;
    }

    /**
     * @param race asettaa rodun
     */
    public void setRace(String race) {
        this.race = race;
    }

    /**
     * @return hakee luokan
     */
    public String getHeroClass() {
        return heroClass;
    }

    /**
     * @param heroClass asettaa luokan
     */
    public void setHeroClass(String heroClass) {
        this.heroClass = heroClass;
    }

    /**
     * @return hakee iän
     */
    public int getAge() {
        return age;
    }

    /**
     * @param age asettaa iän
     */
    public void setAge(int age) {
        this.age = age;
    }

    /**
     * @return hakee tarinan
     */
    public String getLore() {
        return lore;
    }

    /**
     * @param lore asettaa tarinan
     */
    public void setLore(String lore) {
        this.lore = lore;
    }

    /**
     * @param name asettaa nimen
     */
    public void setName(String name) {
        this.name = name;
    }

    private String  name            = "";
    private String  race            = "";
    private String  heroClass       = "";
    private int     age             = 0;
    private String  lore            = "";
    
    private static int nextNumber    = 1;
    
    /**
     * @return the hero's name
     * @example
     * <pre name="test">
     *   Hero batman = new Hero();
     *   batman.helloBatman();
     *   batman.getName() === "Batman";
     * </pre>
     */
    public String getName() {
        return name;
    }
    
    /**
     * Helper method to fill in test values for Batman.
     */
    public void vastaaPitsinNyplays () {
        name = "Batman";
        race = "Human";
        heroClass = "Knight";
        age = 45;
        lore = "Defender of Gotham City";
    }
    
    /**
     * Print the hero's information
     * @param out the output stream to print to
     */
    public void print(PrintStream out) {
        out.println("Name: " + name + "\n" + "Race: " +race + "\n" + "Class: " + heroClass + "\n" + "Age: " + age);
        out.println("Lore: " + lore);
    }
    
    /**
     * Print the hero's information
     * @param os the output stream to print to
     */
    public void print(OutputStream os) {
        print(new PrintStream(os));
    }
    
    /**
     * Assigns the next available registration number to the hero.
     * @return the hero's new hId
     * @example
     * <pre name="test">
     *   Hero batman1 = new Hero();
     *   batman1.getHeroId() === 0;
     *   batman1.registerHeroId();
     *   Hero batman2 = new Hero();
     *   batman2.registerHeroId();
     *   int n1 = batman1.getHeroId();
     *   int n2 = batman2.getHeroId();
     *   n1 === n2-1;
     * </pre>
     */
    public int registerHeroId() {
        hId = nextNumber;
        nextNumber++;
        return hId;
    }
    
    /**
     * Returns the hero's identification number.
     * @return the hero's identification number
     */
    public int getHeroId() {
        return hId;
    }
    
    /**
     * Asettaa numeron ja samalla varmistaa että
     * seuraava numero on aina suurempi kuin tähän mennessä suurin.
     * @param nr asetettava tunnusnumero
     */
    private void setHeroId(int nr) {
        int heroId = nr;
        if (heroId >= nextNumber) nextNumber = heroId + 1;
    }
    
    @Override
    public String toString() {
        return "" +
                getHeroId() + "|" +
                name + "|" +
                race + "|" +
                heroClass + "|" +
                age + "|" +
                lore;
    }
    
    /**
     * Selvitää heron tiedot | erotellusta merkkijonosta
     * (Pitää huolen että seuraavaNro on suurempi kuin tuleva tunnusNro.)???
     * @param rivi josta heron tiedot otetaan
     * 
     * @example
     * <pre name="test">
     *   Hero hero = new Hero();
     *   hero.parse("   3  |  Ankka Aku   | 030201-111C");
     *   hero.getHeroId() === 3;
     *   hero.toString().startsWith("3|Ankka Aku|030201-111C|") === true; // on enemmäkin kuin 3 kenttää, siksi loppu |
     *
     *   hero.registerHeroId();
     *   int n = hero.getHeroId();
     *   hero.parse(""+(n+20));       // Otetaan merkkijonosta vain numero
     *   hero.registerHeroId();           // ja tarkistetaan että seuraavalla kertaa tulee yhtä isompi
     *   hero.getHeroId() === n+20+1;
     *     
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        setHeroId(Mjonot.erota(sb, '|', getHeroId()));
        name = Mjonot.erota(sb, '|', name);
        race = Mjonot.erota(sb, '|', race);
        heroClass = Mjonot.erota(sb, '|', heroClass);
        age = Mjonot.erota(sb, '|', age);
        lore = Mjonot.erota(sb, '|', lore);
    }
    
    
    @Override
    public boolean equals(Object hero) {
        if ( hero == null ) return false;
        return this.toString().equals(hero.toString());
    }


    @Override
    public int hashCode() {
        return hId;
    }


    
    /**
     * Test program for the Hero class.
     * @param args not used
     */
    public static void main(String args[]) {
        Hero archery = new Hero();
        archery.registerHeroId();
        
        archery.vastaaPitsinNyplays();
        archery.print(System.out);

    }

    /**
     * 
     */
    public void rekisteroi() {
        // TODO Auto-generated method stub
        
    }

    /**
     * 
     */
    public void vastaaAkuAnkka() {
        // TODO Auto-generated method stub
        
    }




}
