package HeroList;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

/**
 * @author jessehonkalinna
 * @version Mar 12, 2025
 *
 */
public class Heroes implements Iterable<Hero> {
    private static final int MAX_HEROES = 8;
    private boolean muutettu = false;
    private int count = 0;
    private Hero elements[] = new Hero[MAX_HEROES];

    
    
    /**
     * Adds hero
     * @param hero D
     * @throws StorageException D
     */
    public void add(Hero hero) throws StorageException {
        if ( count >= elements.length ) throw new StorageException("Too many heroes!");
        elements[count++] = hero;
        muutettu = true;
    }
    
    /**
     * @param i saatu parametri
     * @return palauttaa pyydetyt alkiot
     * @throws IndexOutOfBoundsException virheilmoitus
     */
    public Hero anna(int i) throws IndexOutOfBoundsException {
        if ( i < 0 || count <= i ) throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return elements[i];
    }
    
    /**
     * Returns a reference to the i-th hero.
     * @param i the index of the hero to retrieve
     * @return a reference to the hero at index i
     * @throws IndexOutOfBoundsException if i is out of bounds
     */
    public Hero getCount(int i) throws IndexOutOfBoundsException {
        if (i < 0 || count <= i)
            throw new IndexOutOfBoundsException("Illegal index: " + i);
        return elements[i];
    }
    /**
     * Lukee herot tiedostosta
     * @param hakemisto mikä hakemisto
     * @throws StorageException virheen varalla
     */
    public void lueTiedostosta(String hakemisto) throws StorageException {
        String tiedostoNimi = hakemisto + "/heroes.dat";
        count = 0;
        File ftied = new File(tiedostoNimi);
        if (!ftied.exists()) return; // Ei tiedostoa, ei luettavaa

        try (BufferedReader fi = new BufferedReader(new FileReader(tiedostoNimi))) {
            String rivi;
            while ((rivi = fi.readLine()) != null) {
                rivi = rivi.trim();
                if ("".equals(rivi) || rivi.charAt(0) == ';') continue;
                Hero hero = new Hero();
                hero.parse(rivi);
                add(hero);
            }
            muutettu = false;
        } catch (IOException e) {
            throw new StorageException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }
    }



    /**
     * Tallentaa herot tiedostoon
     * @param hakemisto hakemiston nimi
     * @throws StorageException jos epäonnistuu
     */
    public void tallenna(String hakemisto) throws StorageException {
        File dir = new File(hakemisto);
        if (!dir.exists()) dir.mkdirs();

        String tiedostoNimi = hakemisto + "/heroes.dat";
        try (PrintWriter fo = new PrintWriter(new FileWriter(tiedostoNimi, false))) {
            for (int i = 0; i < count; i++) {
                fo.println(elements[i].toString());
            }
            muutettu = false;
        } catch (IOException e) {
            throw new StorageException("Tiedoston " + tiedostoNimi + " kirjoittamisessa ongelmia: " + e.getMessage());
        }
    }

    
    


    /**
     * @return herojen määrä
     */
    public int getCount() {
        return count;
    }
    
    @Override
    public Iterator<Hero> iterator() {
        return new Iterator<Hero>() {
            private int kohdalla = 0;

            @Override
            public boolean hasNext() {
                return kohdalla < count;
            }

            @Override
            public Hero next() {
                if (!hasNext())
                    throw new NoSuchElementException("Ei oo");
                return elements[kohdalla++];
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException("Ei poisteta");
            }
        };
    }
    
    /**
     * Palauttaa kaikki sankarit listana.
     * @return lista sankareista
     */
    public List<Hero> kaikki() {
        List<Hero> lista = new ArrayList<>();
        for (int i = 0; i < count; i++) {
            lista.add(elements[i]);
        }
        return lista;
    }

    /**
     * Etsii sankarit (nyt palauttaa kaikki).
     * @param hakuehto hakuehto (ei käytössä)
     * @param k kenttä (ei käytössä)
     * @return lista löytyneistä sankareista
     */
    public Collection<Hero> etsi(String hakuehto, int k) {
        return kaikki();
    }

}
