package HeroList;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;



/**
 * @author jessehonkalinna
 * @version Apr 10, 2025
 *
 */
public class NamesToAbilities implements Iterable<NameToAbility> {
    private String fileName = "/namesToAbilities.dat";

    private final Collection<NameToAbility> elements = new ArrayList<NameToAbility>();
    
    
    //Constructor
    /**
     * 
     */
    public NamesToAbilities() {
        // not needed now
    }
    
    
    /**
     * Adds names to abilities
     * @param nta jotain
     */
    public void addTo(NameToAbility nta) {
        elements.add(nta);
    }
    
    /**
     * Luetaan tiedostosta
     * @param index kohta tiedostossa
     * @throws StorageException virheenkäsittely
     */
    public void readFromFile(String index) throws StorageException {
        fileName = index + ".nta";
        elements.clear();
        
        try (BufferedReader br = new BufferedReader(new FileReader(fileName))) {
            String line;
            while ((line = br.readLine()) != null) {
                if (line.trim().isEmpty() || line.startsWith(";")) continue; // Ohitetaan tyhjät rivit ja kommentit
                
                NameToAbility nta = new NameToAbility();
                nta.parse(line);
                elements.add(nta);
            }
        } catch (IOException e) {
            throw new StorageException("Virhe tiedoston lukemisessa: " + e.getMessage());
        }
    }
    
    /**
     * Kirjoitetaan tiedostoon
     * @throws StorageException virheenkäsittely
     */
    public void writeToFile() throws StorageException {
        try (BufferedWriter bw = new BufferedWriter(new FileWriter(fileName))) {
            bw.write(";hid|aid|level\n"); // Otsikkorivi
            
            for (NameToAbility nta : elements) {
                bw.write(nta.toString());
                bw.newLine();
            }
        } catch (IOException e) {
            throw new StorageException("Virhe tiedoston tallentamisessa: " + e.getMessage());
        }
    }
    
    /**
     * @return palauttaa alkioiden määrän
     */
    public int getCount() {
        return elements.size();
    }
    
    @Override
    public Iterator<NameToAbility> iterator() {
        return elements.iterator();
    }

    /**
     * @param tunnusnro ei käytössä
     * @return ei käytössä
     */
    public List<NameToAbility> getNamesToAbilities(int tunnusnro) {
        List<NameToAbility> found = new ArrayList<NameToAbility>();
        for (NameToAbility nta : elements)
            if (nta.getHId() == tunnusnro) found.add(nta);
        return found;
    }

    //public static void main(String[] args) {
    //    NamesToAbilities harrasteet = new NamesToAbilities();
    //    NameToAbility pitsi1 = new NameToAbility();
    //    pitsi1.vastaaPitsinNyplays(2);
    //    NameToAbility pitsi2 = new NameToAbility();
    //    pitsi2.vastaaPitsinNyplays(1);
    //    NameToAbility pitsi3 = new NameToAbility();
    //    pitsi3.vastaaPitsinNyplays(2);
    //    NameToAbility pitsi4 = new NameToAbility();
    //    pitsi4.vastaaPitsinNyplays(2);
    //
    //    harrasteet.addTo(pitsi1);
    //    harrasteet.addTo(pitsi2);
    //    harrasteet.addTo(pitsi3);
    //    harrasteet.addTo(pitsi2);
    //    harrasteet.addTo(pitsi4);
    //
    //    System.out.println("============= Harrastukset testi =================");
    //
    //    List<NameToAbility> harrastukset2 = harrasteet.annaHarrastukset(2);
    //
    //    for (NameToAbility har : harrastukset2) {
    //        System.out.print(har.get() + " ");
    //        har.tulosta(System.out);
    //    }
    //
    //}





}
